//======================================================= file = common.h =====
//=  Include file for VOQ switch simulation model                             =
//=============================================================================
//=  Notes:  1) This file contains                                            =
//=               - Constants                                                 =
//=               - Common (external) variables                               =
//=               - Common (external) CSIM types                              =
//=               - Function prototypes                                       =
//=          2) Basic time unit is the cell transmission time                 =
//=---------------------------------------------------------------------------=
//=  History:  KJC (08/12/01) - Genesis                                       =
//=============================================================================
#pragma warn -pro                     // Disable function prototype warning
#pragma warn -stu                     // Disable undefined structure warning
#pragma warn -rch                     // Disable unreachable code warning
#pragma warn -ccc                     // Disable condition always true warning

//----- Conditional definitions -----------------------------------------------
#if defined(MAIN)                    // Determine if main or external
  #define EXTERN
#endif
#if defined(EXTERNAL)
  #define EXTERN extern
#endif

//----- Constants -------------------------------------------------------------
#define VERSION "3.20c <08/12/01>"   // Version number
#define OUT_FILE      "result.txt"   // Name of output file

#define FALSE                   0    // FALSE = 0
#define TRUE                    1    // TRUE = 1

#define CELL_TIME             1.0    // Time to transmit a cell
#define N                      16    // Number of input ports in the switch
#define SCHED_FLAG              1    // 1 = RR, 2 = LQF, 3 = OCF
#define GEN_FLAG                1    // 1 = Bernoulli, 2 = IBP

//----- Overall external variables --------------------------------------------
EXTERN double    Sim_time;            // Simuation time
EXTERN double    Time_stamp[N][N];    // Time stamps for arriving cells
EXTERN int       V_len[N][N];         // Virtual length (for voq_arbit_exp)
EXTERN int       Thresh;              // Threshold for VOQs
EXTERN int       Burst;               // Burst size for VOQs (1 to disable)
EXTERN int       Count[N];            // Burst counter for inport arbiter

//----- External variables for generate1() ------------------------------------
EXTERN double    Lambda;              // Probability of a cell in a slot

//----- External variables for generate2() ------------------------------------
EXTERN double    P;                   // Pr[on at t+1 | on at t]
EXTERN double    Q;                   // Pr[off at t+1 | off at t]
EXTERN double    Alpha;               // Pr[arrival at t | on]

//----- CSIM types ------------------------------------------------------------
EXTERN FACILITY Out_link[N];          // Facility for output port link
EXTERN FACILITY Bus[N];               // Facility for an input port bus
EXTERN EVENT    VOQ_first[N][N];      // Input port VOQ HOL for cells
EXTERN EVENT    VOQ_event[N][N];      // Input port VOQ buffers for cells
EXTERN EVENT    CP_event[N][N];       // Crossbar crosspoint queues for cells
EXTERN EVENT    State_change[N];      // State change in an in_id buffer
EXTERN EVENT    Xfer_voq[N];          // VOQ-to-CP buffer xfer done for in_id
EXTERN EVENT    Xfer_cp[N];           // VOQ-to-CP buffer xfer done for out_id
EXTERN EVENT    Xmit_done[N];         // CP-to-link xmit done event for out_id
EXTERN TABLE    Delay_table[N];       // Delay table for cell delay

//----- Function prototypes ---------------------------------------------------
// Traffic generators
EXTERN void generate1(int in_id);
EXTERN void generate2(int in_id);
EXTERN void (* gen[N])(int in_id);

// Input ports with VOQ (VOQ arbiter in each input port)
EXTERN void inport(int in_id, int out_id, double org_time);
EXTERN void (* in[N])(int in_id, int out_id, double org_time);
EXTERN void voq_arbit_rr(int in_id);
EXTERN void voq_arbit_lqf(int in_id);
EXTERN void voq_arbit_ocf(int in_id);
EXTERN void (* voq_arb[N])(int in_id);

// Output ports (CP arbiter in each output port)
EXTERN void outport(int in_id, int out_id, double org_time);
EXTERN void (* out[N])(int in_id, int out_id, double org_time);
EXTERN void cp_arbit(int out_id);
EXTERN void (* cp_arb[N])(int out_id);

// Run-time simulation status reporter
EXTERN void run_sim(double run_time, int run_num);
