//====================================================== file = outport.c =====
//=  Output port for VOQ switch simulation model                              =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: cp_arbit() - CP arbitration my simple RR                       =
//=            outport() - Output port containing no buffers                  =
//=---------------------------------------------------------------------------=
//=  History:  KJC (08/12/01) - Genesis                                       =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//=============================================================================
//==  Poller for crosspoint buffers                                          ==
//=============================================================================
void cp_arbit(int out_id)
{
  int      in_id;               // Input port id
  int      in_poll;             // Input port id to be polled
  int      flag;                // A flag variable
  int      i;                   // Loop counter

  create("cp_arbit");

  // Poll output port CP buffers forever
  in_id = 0;
  while(1)
  {
    // Test if any CP buffers are non-empty
    flag = 0;
    for (i=0; i<N; i++)
      if (queue_cnt(CP_event[i][out_id]) > 0) flag = 1;

    // If all CP buffers are empty, wait for a transfer into a CP buffer
    if (flag == 0) wait(Xfer_cp[out_id]);

      // Poll one "full circle"
    for (i=0; i<N; i++)
    {
      in_poll = (in_id + i) % N;
      if ((queue_cnt(CP_event[in_poll][out_id]) > 0) &&
          (status(Out_link[out_id]) == FREE))
      {
        set(CP_event[in_poll][out_id]);
        wait(Xmit_done[out_id]);
        in_id = (in_poll + 1) % N;
        hold(0.0);
        break;
      }
    }
  }
}

//=============================================================================
//==  Output port "owning" the CP buffers                                    ==
//=============================================================================
void outport(int in_id, int out_id, double org_time)
{
  create("outport");

  // Queue the arriving cell in the appropriate crosspoint buffer
  queue(CP_event[in_id][out_id]);

  // Set the state change for allowing the next cell to be transferred in
  set(State_change[in_id]);

  // Transmit the cell and record the delay and byte count
  assert(status(Out_link[out_id]) == FREE);
  reserve(Out_link[out_id]);
  hold(CELL_TIME);
  release(Out_link[out_id]);
  record((clock - org_time), Delay_table[out_id]);

  // Set transmit done event
  set(Xmit_done[out_id]);

  return;
}
